
(object Petal
    version    	45
    _written   	"Rose 7.5.0103.1920"
    charSet    	0)

(object Class_Category "CLIPS Interface"
    is_unit    	TRUE
    is_loaded  	TRUE
    quid       	"3BD772E00243"
    documentation 	"The CLIPS Interface Domain is responsible for implementing the interface to the CLIPS Inference Engine.  This domain provides wrapper classes to insulate clients from the low-level features of CLIPS."
    visible_categories 	(list visibility_relationship_list
	(object Visibility_Relationship
	    quid       	"3BE2086B0297"
	    supplier   	"Logical View::Standard Template Library"
	    quidu      	"3BE0B7CA0228"))
    exportControl 	"Public"
    logical_models 	(list unit_reference_list
	(object Class "CLIPS Construct"
	    quid       	"3BD775240388"
	    documentation 	"CLIPS Construct is an abstract class of CLIPS core.  This abstract base class functions as an interface for the commands which may be held within the memory of the caller."
	    operations 	(list Operations
		(object Operation "Execute"
		    quid       	"3BD77A960007"
		    documentation 	
|Once the command string is built up, this method sends command string to Route_Command method.
|
|PDL:
|  Finish building command string
|  Route_Command(cmdString);
|
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Create"
		    quid       	"3BE22B330327"
		    documentation 	"Create corresponds to a constructor, which constructs a CLIPS class."
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Route_Command"
		    quid       	"3BE22C0A03B2"
		    documentation 	"This function calls CLIPS to execute a command defined in command string."
		    concurrency 	"Sequential"
		    opExportControl 	"Protected"
		    uid        	0)))
	(object Class "CLIPS_Class"
	    quid       	"3BD775330307"
	    documentation 	"CLIPS_Class is a sub-class of CLIPS_Construct, which is used to define CLIPS classes corresponding to Concept Types and Conceptual Relation Types in a KnowledgeBase."
	    superclasses 	(list inheritance_relationship_list
		(object Inheritance_Relationship
		    quid       	"3BD7754E01ED"
		    supplier   	"Logical View::CLIPS Interface::CLIPS Construct"
		    quidu      	"3BD775240388"))
	    operations 	(list Operations
		(object Operation "Add_Slot"
		    quid       	"3BE22AA201A2"
		    documentation 	
|Add one slot to a defined class.
|
|PDL:
|         Slots.insert(slot_name)
|
		    
		    parameters 	(list Parameters
			(object Parameter "slot_name"
			    quid       	"3BE48909003F"))
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Delete_Slot"
		    quid       	"3BE22AB101B7"
		    documentation 	"Delete one slot from a defined class."
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0))
	    class_attributes 	(list class_attribute_list
		(object ClassAttribute "m_slots"
		    quid       	"3BE3617B02A3")))
	(object Class "CLIPS_Instance"
	    quid       	"3BD7753C03C8"
	    documentation 	"CLIPS_Instance is a sub-class of CLIPS_Construct, which creates instances of a defined class in CLIPS."
	    superclasses 	(list inheritance_relationship_list
		(object Inheritance_Relationship
		    quid       	"3BD7755101A2"
		    supplier   	"Logical View::CLIPS Interface::CLIPS Construct"
		    quidu      	"3BD775240388"))
	    operations 	(list Operations
		(object Operation "Make"
		    quid       	"3BE235810384"
		    documentation 	
|Make an instance for a defined CLIPS class.
|
|PDL:
|   call CLIPS MakeInstance method
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Unmake"
		    quid       	"3BE22FB701E8"
		    documentation 	
|Remove an instance of a CLIPS class.
|
|PDL:
|    call CLIPS UnmakeInstance(instancePtr) method
|
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Send"
		    quid       	"3BE236160234"
		    documentation 	"Sends a message to an instance using the CLIPS send method."
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Set_Value"
		    quid       	"3BE361C003E3"
		    documentation 	
|Set the value of the slotname before making an instance.
|
|PDL:
|  Name_Value_Pairs.name = Value;
		    
		    parameters 	(list Parameters
			(object Parameter "Name"
			    quid       	"3BE361D20045"
			    type       	"String"
			    quidu      	"3BE21D8E007B")
			(object Parameter "Value"
			    quid       	"3BE361F303C8"
			    type       	"String"
			    quidu      	"3BE21D8E007B"))
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0))
	    class_attributes 	(list class_attribute_list
		(object ClassAttribute "m_name_value_pairs"
		    quid       	"3BE363B00291")))
	(object Class "CLIPS_Rule"
	    quid       	"3BD77543035A"
	    documentation 	"CLIPS_Rule is a sub-class of CLIPS_Contruct, which defines all the canonical rules of conceptual graphs and defines rules to implement the command given by the client."
	    superclasses 	(list inheritance_relationship_list
		(object Inheritance_Relationship
		    quid       	"3BD775540188"
		    supplier   	"Logical View::CLIPS Interface::CLIPS Construct"
		    quidu      	"3BD775240388"))
	    operations 	(list Operations
		(object Operation "Define"
		    quid       	"3BE3631F03AB"
		    documentation 	
|Define CLIPS rules.
|
|PDL:
|  Create defrule Command String
|  Call Route_Command
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Undefine"
		    quid       	"3BE3630D025B"
		    documentation 	
|Remove CLIPS rule.
|
|PDL:
|  Call CLIPS undefine method
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)))
	(object Class "CLIPS_Controller"
	    quid       	"3BD7755F038C"
	    documentation 	"The CLIPS_Controller is responsible for performing activities to control the entire CLIPS environment."
	    operations 	(list Operations
		(object Operation "Initialize"
		    quid       	"3BD77A6B00A5"
		    documentation 	
|Initializes the CLIPS system. Must be called prior to any other
|CLIPS function call. 
|
|PDL:
|  if this is the first invocation of Initialize
|    call clips InitializeEnvironment method
|    set m_initialized to True
|
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Run"
		    quid       	"3BE2343A0045"
		    documentation 	
|Allows rules to execute.
|
|PDL:
|  Call the CLIPS run (runLimit) method
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Reset"
		    quid       	"3BE234450326"
		    documentation 	
|Resets the CLIPS environment.
|
|PDL:
|  Call CLIPS Reset method
|
		    
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0)
		(object Operation "Clear"
		    quid       	"3BE2344C0127"
		    documentation 	"Clear the CLIPS environment."
		    concurrency 	"Sequential"
		    opExportControl 	"Public"
		    uid        	0))
	    class_attributes 	(list class_attribute_list
		(object ClassAttribute "m_initialized"
		    quid       	"3BE48B5301A0"
		    type       	"Boolean"
		    initv      	"False"))))
    logical_presentations 	(list unit_reference_list
	(object ClassDiagram "Main"
	    quid       	"3BD773BE0397"
	    title      	"Main"
	    zoom       	100
	    max_height 	28350
	    max_width  	21600
	    origin_x   	0
	    origin_y   	0
	    items      	(list diagram_item_list
		(object ClassView "Class" "Logical View::CLIPS Interface::CLIPS Construct" @1
		    ShowCompartmentStereotypes 	TRUE
		    IncludeAttribute 	TRUE
		    IncludeOperation 	TRUE
		    location   	(1019, 322)
		    font       	(object Font
			size       	10
			face       	"Arial"
			bold       	FALSE
			italics    	FALSE
			underline  	FALSE
			strike     	FALSE
			color      	0
			default_color 	TRUE)
		    label      	(object ItemLabel
			Parent_View 	@1
			location   	(824, 166)
			fill_color 	13434879
			nlines     	2
			max_width  	390
			justify    	0
			label      	"CLIPS Construct")
		    icon_style 	"Icon"
		    line_color 	3342489
		    fill_color 	13434879
		    quidu      	"3BD775240388"
		    width      	408
		    height     	336
		    annotation 	8
		    autoResize 	TRUE)
		(object ClassView "Class" "Logical View::CLIPS Interface::CLIPS_Class" @2
		    ShowCompartmentStereotypes 	TRUE
		    IncludeAttribute 	TRUE
		    IncludeOperation 	TRUE
		    location   	(565, 776)
		    font       	(object Font
			size       	10
			face       	"Arial"
			bold       	FALSE
			italics    	FALSE
			underline  	FALSE
			strike     	FALSE
			color      	0
			default_color 	TRUE)
		    label      	(object ItemLabel
			Parent_View 	@2
			location   	(422, 620)
			fill_color 	13434879
			nlines     	2
			max_width  	286
			justify    	0
			label      	"CLIPS_Class")
		    icon_style 	"Icon"
		    line_color 	3342489
		    fill_color 	13434879
		    quidu      	"3BD775330307"
		    width      	304
		    height     	336
		    annotation 	8
		    autoResize 	TRUE)
		(object ClassView "Class" "Logical View::CLIPS Interface::CLIPS_Instance" @3
		    ShowCompartmentStereotypes 	TRUE
		    IncludeAttribute 	TRUE
		    IncludeOperation 	TRUE
		    location   	(1075, 828)
		    font       	(object Font
			size       	10
			face       	"Arial"
			bold       	FALSE
			italics    	FALSE
			underline  	FALSE
			strike     	FALSE
			color      	0
			default_color 	TRUE)
		    label      	(object ItemLabel
			Parent_View 	@3
			location   	(806, 628)
			fill_color 	13434879
			nlines     	2
			max_width  	539
			justify    	0
			label      	"CLIPS_Instance")
		    icon_style 	"Icon"
		    line_color 	3342489
		    fill_color 	13434879
		    quidu      	"3BD7753C03C8"
		    width      	557
		    height     	425
		    annotation 	8)
		(object ClassView "Class" "Logical View::CLIPS Interface::CLIPS_Rule" @4
		    ShowCompartmentStereotypes 	TRUE
		    IncludeAttribute 	TRUE
		    IncludeOperation 	TRUE
		    location   	(1541, 787)
		    font       	(object Font
			size       	10
			face       	"Arial"
			bold       	FALSE
			italics    	FALSE
			underline  	FALSE
			strike     	FALSE
			color      	0
			default_color 	TRUE)
		    label      	(object ItemLabel
			Parent_View 	@4
			location   	(1424, 656)
			fill_color 	13434879
			nlines     	2
			max_width  	234
			justify    	0
			label      	"CLIPS_Rule")
		    icon_style 	"Icon"
		    line_color 	3342489
		    fill_color 	13434879
		    quidu      	"3BD77543035A"
		    width      	252
		    height     	286
		    annotation 	8
		    autoResize 	TRUE)
		(object InheritTreeView "" @5
		    location   	(1019, 581)
		    line_color 	3342489
		    fill_color 	13434879
		    supplier   	@1
		    vertices   	(list Points
			(1019, 581)
			(1019, 490)))
		(object InheritView "" @6
		    stereotype 	TRUE
		    line_color 	3342489
		    quidu      	"3BD7754E01ED"
		    client     	@2
		    supplier   	@1
		    line_style 	3
		    origin_attachment 	(651, 608)
		    terminal_attachment 	(651, 581)
		    drawSupplier 	@5)
		(object InheritView "" @7
		    stereotype 	TRUE
		    line_color 	3342489
		    quidu      	"3BD7755101A2"
		    client     	@3
		    supplier   	@1
		    line_style 	3
		    origin_attachment 	(1066, 615)
		    terminal_attachment 	(1066, 581)
		    drawSupplier 	@5)
		(object InheritView "" @8
		    stereotype 	TRUE
		    line_color 	3342489
		    quidu      	"3BD775540188"
		    client     	@4
		    supplier   	@1
		    line_style 	3
		    origin_attachment 	(1497, 644)
		    terminal_attachment 	(1497, 581)
		    drawSupplier 	@5)
		(object ClassView "Class" "Logical View::CLIPS Interface::CLIPS_Controller" @9
		    ShowCompartmentStereotypes 	TRUE
		    IncludeAttribute 	TRUE
		    IncludeOperation 	TRUE
		    location   	(337, 319)
		    font       	(object Font
			size       	10
			face       	"Arial"
			bold       	FALSE
			italics    	FALSE
			underline  	FALSE
			strike     	FALSE
			color      	0
			default_color 	TRUE)
		    label      	(object ItemLabel
			Parent_View 	@9
			location   	(31, 113)
			fill_color 	13434879
			nlines     	2
			max_width  	612
			justify    	0
			label      	"CLIPS_Controller")
		    icon_style 	"Icon"
		    line_color 	3342489
		    fill_color 	13434879
		    quidu      	"3BD7755F038C"
		    width      	630
		    height     	436
		    annotation 	8
		    autoResize 	TRUE)))))
